﻿using Syncfusion.ListView.XForms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Emit;
using System.Text;

using Xamarin.Forms;

namespace Swiping
{
    public class SwipingPageCS : ContentPage
    {
        #region Fields

        Image leftImage;
        Image rightImage;
        int itemIndex = -1;
        SfListView listView;
        ListViewSwipingViewModel viewModel;

        #endregion

        #region Constructor

        public SwipingPageCS()
        {
            InitializeListView();
            Content = listView;
            Title = "SfListView Swiping Sample";
        }

        #endregion

        #region Private Methods

        private void InitializeListView()
        {
            listView = new SfListView();
            viewModel = new ListViewSwipingViewModel();
            listView.ItemsSource = viewModel.InboxInfo;
            listView.ItemTemplate = new DataTemplate(() => { return CreateItemTemplate(); });
            listView.LeftSwipeTemplate = new DataTemplate(() => { return CreateLeftSwipeTemplate(); });
            listView.RightSwipeTemplate = new DataTemplate(() => { return CreateRightSwipeTemplate(); });
            listView.ItemSize = 100;
            listView.SwipeOffset = 120;
            listView.SwipeThreshold = 30;
            listView.HeaderSize = 70;
            listView.IsStickyHeader = true;
            listView.AllowSwiping = true;
            listView.SwipeStarted += ListView_SwipeStarted;
            listView.SwipeEnded += ListView_SwipeEnded;
            listView.HeaderTemplate = new DataTemplate(() =>
            {
                var grid = new Grid() { BackgroundColor = Color.FromHex("#4CA1FE"), HeightRequest = 45 };

                var headerlabel = new Label
                {
                    TextColor = Color.White,
                    FontSize = 18,
                    Margin = new Thickness(10),
                    VerticalOptions = LayoutOptions.Center,
                    HorizontalOptions = LayoutOptions.Center,
                    LineBreakMode = LineBreakMode.NoWrap,
                    FontAttributes = FontAttributes.Bold,
                    Text = "Inbox"
                };

                grid.Children.Add(headerlabel);

                return grid;
            });
        }

        private Grid CreateItemTemplate()
        {
            var templategrid = new Grid();

            var rowdefinition0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var rowdefinition1 = new RowDefinition { Height = 1 };
            templategrid.RowDefinitions.Add(rowdefinition0);
            templategrid.RowDefinitions.Add(rowdefinition1);

            var grid = new Grid();
            var columndefinition0 = new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) };
            var columndefinition1 = new ColumnDefinition { Width = new GridLength(1, GridUnitType.Auto) };
            grid.ColumnDefinitions.Add(columndefinition0);
            grid.ColumnDefinitions.Add(columndefinition1);

            var contentgrid = new Grid() { Padding = new Thickness(5) };
            var row0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var row1 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var row2 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            contentgrid.RowDefinitions.Add(row0);
            contentgrid.RowDefinitions.Add(row1);
            contentgrid.RowDefinitions.Add(row2);

            var titlelabel = new Label();
            titlelabel.SetBinding(Label.TextProperty, new Binding("Title"));
            titlelabel.LineBreakMode = LineBreakMode.NoWrap;
            titlelabel.FontAttributes = FontAttributes.Bold;
            titlelabel.TextColor = Color.FromHex("#474747");
            titlelabel.FontSize = 20;

            var subjectlabel = new Label();
            subjectlabel.FontSize = 16;
            subjectlabel.LineBreakMode = LineBreakMode.NoWrap;
            subjectlabel.SetBinding(Label.TextProperty, new Binding("Subject"));
            subjectlabel.TextColor = Color.FromHex("#474747");

            var descriptionlabel = new Label();
            descriptionlabel.FontSize = 12;
            descriptionlabel.LineBreakMode = LineBreakMode.NoWrap;
            descriptionlabel.SetBinding(Label.TextProperty, new Binding("Description"));
            descriptionlabel.TextColor = Color.FromHex("#474747");

            contentgrid.Children.Add(titlelabel, 0, 0);
            contentgrid.Children.Add(subjectlabel, 0, 1);
            contentgrid.Children.Add(descriptionlabel, 0, 2);

            var imagegrid = new Grid() { Padding = new Thickness(5, 10, 5, 10), HorizontalOptions = LayoutOptions.End };
            var definition0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var definition1 = new RowDefinition { Height = 30 };
            imagegrid.RowDefinitions.Add(definition0);
            imagegrid.RowDefinitions.Add(definition1);

            var datelabel = new Label();
            datelabel.FontSize = 12;
            datelabel.LineBreakMode = LineBreakMode.NoWrap;
            datelabel.SetBinding(Label.TextProperty, new Binding("Date"));
            datelabel.TextColor = Color.FromHex("#474747");

            var normalimage = new Image() { HorizontalOptions = LayoutOptions.Center, HeightRequest = 30, WidthRequest = 30, VerticalOptions = LayoutOptions.Center };
            normalimage.SetBinding(Image.SourceProperty, new Binding("IsFavorite")
            {
                Converter = new SwipingBoolToImageConverter()
            });

            imagegrid.Children.Add(datelabel, 0, 0);
            imagegrid.Children.Add(normalimage, 0, 1);

            grid.Children.Add(contentgrid);
            grid.Children.Add(imagegrid, 1, 0);

            var frame = new Frame() { OutlineColor = Color.FromHex("#E4E4E4"), HasShadow = true, HeightRequest = 1 };

            templategrid.Children.Add(grid);
            templategrid.Children.Add(frame, 0, 1);

            return templategrid;
        }

        private Grid CreateLeftSwipeTemplate()
        {
            var grid = new Grid();
            var columndefinition0 = new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) };
            var columndefinition1 = new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) };
            grid.ColumnDefinitions.Add(columndefinition0);
            grid.ColumnDefinitions.Add(columndefinition1);

            var templategrid1 = new Grid() { BackgroundColor = Color.FromHex("#009EDA"), HorizontalOptions = LayoutOptions.Fill, VerticalOptions = LayoutOptions.Fill };
            var favouritegrid = new Grid() { HorizontalOptions = LayoutOptions.Center, VerticalOptions = LayoutOptions.Center };
            var favouriteImage = new Image() { BackgroundColor = Color.Transparent, HeightRequest = 35, WidthRequest = 35 };
            favouriteImage.BindingContextChanged += FavouriteImage_BindingContextChanged;
            favouriteImage.Source = ImageSource.FromResource("Swiping.Images.Favorites.png");
            favouritegrid.Children.Add(favouriteImage);
            templategrid1.Children.Add(favouritegrid);

            var templategrid2 = new Grid() { BackgroundColor = Color.FromHex("#DC595F"), HorizontalOptions = LayoutOptions.Fill, VerticalOptions = LayoutOptions.Fill };
            var deletegrid = new Grid() { HorizontalOptions = LayoutOptions.Center, VerticalOptions = LayoutOptions.Center };
            var deleteImage = new Image() { BackgroundColor = Color.Transparent, HeightRequest = 35, WidthRequest = 35 };
            deleteImage.BindingContextChanged += DeleteImage_BindingContextChanged;
            deleteImage.Source = ImageSource.FromResource("Swiping.Images.Delete.png");
            deletegrid.Children.Add(deleteImage);
            templategrid2.Children.Add(deletegrid);

            grid.Children.Add(templategrid1);
            grid.Children.Add(templategrid2, 1, 0);

            return grid;
        }

        private Grid CreateRightSwipeTemplate()
        {
            var grid = new Grid() { BackgroundColor = Color.FromHex("#DC595F"), HorizontalOptions = LayoutOptions.Fill, VerticalOptions = LayoutOptions.Fill };
            var deletegrid = new Grid() { HorizontalOptions = LayoutOptions.Center, VerticalOptions = LayoutOptions.Center };
            var deleteImage = new Image() { BackgroundColor = Color.Transparent, HeightRequest = 35, WidthRequest = 35 };
            deleteImage.BindingContextChanged += DeleteImage_BindingContextChanged;
            deleteImage.Source = ImageSource.FromResource("Swiping.Images.Delete.png");
            deletegrid.Children.Add(deleteImage);
            grid.Children.Add(deletegrid);

            return grid;
        }

        private void SetFavorites()
        {
            if (itemIndex >= 0)
            {
                var item = viewModel.InboxInfo[itemIndex];
                item.IsFavorite = !item.IsFavorite;
            }
            this.listView.ResetSwipe();
        }

        private void Delete()
        {
            if (itemIndex >= 0)
                viewModel.InboxInfo.RemoveAt(itemIndex);
            this.listView.ResetSwipe();
        }

        #endregion

        #region CallBacks

        private void ListView_SwipeEnded(object sender, SwipeEndedEventArgs e)
        {
            if (e.SwipeOffset >= 360)
            {
                if (e.ItemIndex >= 0)
                    viewModel.InboxInfo.RemoveAt(e.ItemIndex);
                e.SwipeOffset = 0;
            }
            itemIndex = e.ItemIndex;
        }

        private void ListView_SwipeStarted(object sender, SwipeStartedEventArgs e)
        {
            itemIndex = -1;
        }

        private void DeleteImage_BindingContextChanged(object sender, EventArgs e)
        {
            if (rightImage == null)
            {
                rightImage = sender as Image;
                (rightImage.Parent as View).GestureRecognizers.Add(new TapGestureRecognizer() { Command = new Command(Delete) });
                rightImage.Source = ImageSource.FromResource("Swiping.Images.Delete.png");
            }
        }

        private void FavouriteImage_BindingContextChanged(object sender, EventArgs e)
        {
            if (leftImage == null)
            {
                leftImage = sender as Image;
                (leftImage.Parent as View).GestureRecognizers.Add(new TapGestureRecognizer() { Command = new Command(SetFavorites) });
                leftImage.Source = ImageSource.FromResource("Swiping.Images.Favorites.png");
            }
        }

        #endregion
    }
}
